/*
 * @(#)ChargedFluidTool.java		1.10 07/01/20
 *
 * ChargedFluid package
 *
 * COPYRIGHT NOTICE
 * Copyright (c) 2007 Herbert H.H. Chang, Daniel J. Valentino, Gary R. Duckwiler, and Arthur W. Toga
 * Laboratory of Neuro Imaging, Department of Neurology, UCLA.
 */

package tools;

import java.awt.Rectangle;
import java.awt.Color;
import java.awt.image.BufferedImage;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.util.Iterator;
import java.awt.Point;
import org.medtoolbox.jviewbox.viewport.Viewport;
import org.medtoolbox.jviewbox.viewport.ViewportCluster;
import org.medtoolbox.jviewbox.viewport.ViewportTool;
import org.medtoolbox.jviewbox.viewport.annotation.DynamicAnnotationShape;

/**
 * Tool for segmenting an image in a viewport. When the left mouse button is pressed,
 * a seed point is initialized. The size of initial contours can be changed using the
 * middle and right buttons.
 */
public class ChargedFluidTool extends ViewportTool
{
	/** 
	 * Computation function. 
	 */
	private Matrix _matrix;
	
	/** 
	 * ViewportCluster for displaying the viewports. 
	 */
	private ViewportCluster _vpc;
	
	/** 
	 * Viewport for storing the image. 
	 */
	private Viewport _origViewport;
	
	/** 
	 * The width of the image. 
	 */
	private int _imageWidth;
	
	/**
	 * The height of the image. 
	 */
	private int _imageHeight;
	
	/**
	 * The start postion of selected image. 
	 */
	private int _xStart, _yStart;
	
	/**
	 * The width of the selected area. 
	 */
	private int _width;
	
	/** 
	 * The height of the selected area. 
	 */
	private int _height;
	
	/**
	 * Annotation dynamic shape. 
	 */
	private DynamicAnnotationShape _dyShape;
	
	/**
	 * Cursor point. 
	 */
	private Point _cursorPoint = new Point();
	
	/**
	 * Geometry contour for the inital placement. 
	 */
	private GeometryContour _geometryContour;
	
	/**
	 * Boolean indicating if the initial contour is square or not. 
	 */
	private boolean _isSquare;
  
  
	/** 
	 * Constructs a ChargedFluidTool with the given <code>Matrix</code>.
	 * @param	matrix	the core part of handling the computation.
	 */
	public ChargedFluidTool(Matrix matrix)
	{
		super("ChargedFluid Tool", "Place seed point", "Decrease contour size", "Increase contour size");	
		_matrix = matrix;
		_imageWidth = _matrix.getImageWidth();
		_imageHeight = _matrix.getImageHeight();
		_width = _matrix.getInitialBoxWidth();
		_height = _matrix.getInitialBoxHeight();
		_isSquare = _matrix.isSquare();
	}
  
  
	/**
	* Invoked when a mouse button is moved on a ViewportCluster.
	*
	* @param vpc ViewportCluster which received the event.
	* @param vp Viewport which the mouse cursor was on during the event.
	* @param e Event created by the button press.
	*/
	public void mouseMoved(ViewportCluster vpc, Viewport vp, MouseEvent e)
	{
		Iterator it = vpc.getViewports().iterator();
		_origViewport = (Viewport)it.next();
		_origViewport = (Viewport)it.next();
		_isSquare = _matrix.isSquare();
		
		try {
			if (vp == _origViewport) {
				vp.removeAnnotation(_dyShape);
				// Transform the mouse point to annotation coordinates.
				AffineTransform annotationTransform = vp.getViewportTransform();
				annotationTransform.concatenate(vp.getImageTransform());
				annotationTransform.inverseTransform(e.getPoint(), _cursorPoint);
				// If the initial contour is square
				if (_isSquare) {
					_xStart = _cursorPoint.x - _width/2;
					_yStart = _cursorPoint.y - _height/2;
					// Create a sqaure shaped contour.
					_geometryContour = new GeometryContour(_xStart, _yStart, _width, _height);
					_dyShape = new DynamicAnnotationShape(_geometryContour.rectangle);
				}
				// If the initial contour is not square but circle.
				else {
					// Set the radius be the minimum of width and height.
					int radius = Math.min(_width, _height);
					radius /= 2;
					// Create a circle-like contour.
					_geometryContour = new GeometryContour(_cursorPoint, radius);
					_dyShape = new DynamicAnnotationShape(_geometryContour.circlePolygon);
				}
				_dyShape.setForegroundColor(Color.yellow);
				_dyShape.setBackgroundColor(null);
				vp.addAnnotation(_dyShape);
				// Repaint the Viewport
				vpc.repaint(vp);
			}
		}
		catch (Exception ex) {
			System.out.println("Error:  " + ex.getMessage());
			ex.printStackTrace();
		}
	}
  
  
	/**
	* Invoked when a mouse button is pressed on a ViewportCluster.
	*
	* @param vpc ViewportCluster which received the event.
	* @param vp Viewport which the mouse cursor was on during the event.
	* @param e Event created by the button press.
	* @param button The button that was pressed.
	*/
	public void mousePressed(ViewportCluster vpc, Viewport vp, MouseEvent e, int button)
	{
		Iterator it = vpc.getViewports().iterator();
		_origViewport = (Viewport)it.next();
		_origViewport = (Viewport)it.next();
		
		// Left button: Initialize seed points
		if (button == ViewportTool.LEFT_MOUSE_BUTTON && _origViewport == vp) {
			try {
				// Transform the mouse point to image coordinates.
				Point cursorPoint = new Point();
				AffineTransform imageTransform = vp.getViewportTransform();
				imageTransform.concatenate(vp.getImageTransform());
				imageTransform.inverseTransform(e.getPoint(), cursorPoint);
								
				// Create a Charged Fluid
				_matrix.createChargedFluid(_geometryContour);
				// Save the seed point.
				_matrix.addSeedPoint(cursorPoint);
			
				// Repaint the Viewport
				vpc.repaint(_origViewport);
			}
			catch (Exception ex) {
				System.out.println("Error:  " + ex.getMessage());
				ex.printStackTrace();
			}	
		}
		// Right button: Increase the box size by twofold.
		if (button == ViewportTool.RIGHT_MOUSE_BUTTON && _origViewport == vp) {
			// If the box size is the maximum limit, jump to the minimum size.
			if ((_width == _imageWidth/2) && (_height == _imageHeight/2)) {
				_width = 2;
				_height = 2;
			}
			else {
				_width *= 2;
				_height *= 2;
			}
			_origViewport.removeAnnotation(_dyShape);
			// If the initial contour is square
			if (_isSquare) {
				_xStart = _cursorPoint.x - _width/2;
				_yStart = _cursorPoint.y - _height/2;
				// Create a sqaure shaped contour.
				_geometryContour = new GeometryContour(_xStart, _yStart, _width, _height);
				_dyShape = new DynamicAnnotationShape(_geometryContour.rectangle);
			}
			// If the initial contour is not square but circle.
			else {
				// Set the radius be the minimum of width and height.
				int radius = Math.min(_width, _height);
				radius /= 2;
				// Create a circle-like contour.
				_geometryContour = new GeometryContour(_cursorPoint, radius);
				_dyShape = new DynamicAnnotationShape(_geometryContour.circlePolygon);
			}
			_dyShape.setForegroundColor(Color.yellow);
			_dyShape.setBackgroundColor(null);
			_origViewport.addAnnotation(_dyShape);
			vpc.repaint(vp);  
		}
		// Middle button: Decrease the box size by twofold.
		if (button == ViewportTool.MIDDLE_MOUSE_BUTTON && _origViewport == vp) {
			// If the box size is the minimum limit, jump to the maximum size.
			if ((_width == 2) && (_height == 2)) {
				_width = _imageWidth/2;
				_height = _imageHeight/2;
			}
			else {
				_width /= 2;
				_height /= 2;
			}
			_origViewport.removeAnnotation(_dyShape);
			// If the initial contour is square
			if (_isSquare) {
				_xStart = _cursorPoint.x - _width/2;
				_yStart = _cursorPoint.y - _height/2;
				// Create a sqaure shaped contour.
				_geometryContour = new GeometryContour(_xStart, _yStart, _width, _height);
				_dyShape = new DynamicAnnotationShape(_geometryContour.rectangle);
			}
			// If the initial contour is not square but circle.
			else {
				// Set the radius be the minimum of width and height.
				int radius = Math.min(_width, _height);
				radius /= 2;
				// Create a circle-like contour.
				_geometryContour = new GeometryContour(_cursorPoint, radius);
				_dyShape = new DynamicAnnotationShape(_geometryContour.circlePolygon);
			}
			_dyShape.setForegroundColor(Color.yellow);
			_dyShape.setBackgroundColor(null);
			_origViewport.addAnnotation(_dyShape);
			vpc.repaint(vp);  
		}
	}
  
  
	/**
	* Invoked when a mouse is moved outside a ViewportCluster.
	*
	* @param vpc ViewportCluster which received the event.
	* @param vp Viewport which the mouse cursor was on during the event.
	* @param e Event created by the button press.
	*/
	public void mouseExited(ViewportCluster vpc, Viewport vp, MouseEvent e)
	{
		_origViewport.removeAnnotation(_dyShape);
		vpc.repaint(_origViewport);
	}
}