/*
 * @(#)SelectTool.java		1.10 07/01/20
 *
 * ChargedFluid package
 *
 * COPYRIGHT NOTICE
 * Copyright (c) 2007 Herbert H.H. Chang, Daniel J. Valentino, Gary R. Duckwiler, and Arthur W. Toga
 * Laboratory of Neuro Imaging, Department of Neurology, UCLA.
 */

package tools;

import java.awt.Point;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.Raster;
import java.awt.Graphics2D;
import java.awt.AlphaComposite;
import java.awt.geom.AffineTransform;
import java.util.Iterator;
import java.awt.event.MouseEvent;
import org.medtoolbox.jviewbox.viewport.Viewport;
import org.medtoolbox.jviewbox.viewport.ViewportCluster;
import org.medtoolbox.jviewbox.viewport.ViewportTool;
import org.medtoolbox.jviewbox.viewport.annotation.DynamicAnnotationShape;


/**
 * Tool for selecting an image inside the first Viewport to be processed in the second
 * viewport. When the left mouse button is pressed on the image, the image is duplicated and
 * shown in the second viewport in RGB format.
 */
 
public class SelectTool extends ViewportTool
{
  /** Viewport for storing the image. */
  private Viewport _origViewport;
  /** The width of the image. */
  private int _imageWidth;
  /** The height of the image. */
  private int _imageHeight;
  /** The start postion of selected image. */
  private int _xStart, _yStart;
  /** The width of selected area. */
  private int _width;
  /** The height of selected area. */
  private int _height;
  /** Annotation dynamic shape. */
  private DynamicAnnotationShape _dyShape;
  /** Cursor point. */
  private Point _cursorPoint = new Point();
  /** Computation function. */
  private Matrix _matrix;	
	
	
  /** Constructs a SelectTool. */
  public SelectTool(Matrix matrix)
  {
  	super("Select Tool", "Select an Image", "", "");
  	
  	_matrix = matrix;
  	// Get the image size
  	_imageWidth = _matrix.getImageWidth();
    _imageHeight = _matrix.getImageHeight();
    // Set the initial area size.
  	_width = _imageWidth;
  	_height = _imageHeight;
  }
  
  
  /**
   * Invoked when a mouse button is moved on a ViewportCluster.
   *
   * @param vpc ViewportCluster which received the event.
   * @param vp Viewport which the mouse cursor was on during the event.
   * @param e Event created by the button press.
   */
  public void mouseMoved(ViewportCluster vpc, Viewport vp, MouseEvent e)
  {
    Rectangle rect;
    Iterator it = vpc.getViewports().iterator();
	_origViewport = (Viewport)it.next();
    
    try {
      if (vp == _origViewport) {
      	vp.removeAnnotation(_dyShape);
      	// Transform the mouse point to annotation coordinates.
      	AffineTransform annotationTransform = vp.getViewportTransform();
      	annotationTransform.concatenate(vp.getImageTransform());
      	annotationTransform.inverseTransform(e.getPoint(), _cursorPoint);
      	_xStart = _cursorPoint.x - _width/2;
      	_yStart = _cursorPoint.y - _height/2;
      	// Set the origin of the rectangle zero if it is the size of the image.
      	if ((_width == _imageWidth) && (_height == _imageHeight)) {
      		_xStart = 0;
      		_yStart = 0;	
      	}
      	rect = new Rectangle(_xStart, _yStart, _width, _height);
      	_dyShape = new DynamicAnnotationShape(rect);
      	_dyShape.setForegroundColor(Color.yellow);
      	_dyShape.setBackgroundColor(null);
      	vp.addAnnotation(_dyShape);
      	// Repaint the Viewport
      	vpc.repaint(vp);
      }
    }
    catch (Exception ex) {
	    System.out.println("Error:  " + ex.getMessage());
	    ex.printStackTrace();
    }
  }
  
  
  /**
   * Invoked when a mouse button is pressed on a ViewportCluster.
   *
   * @param vpc ViewportCluster which received the event.
   * @param vp Viewport which the mouse cursor was on during the event.
   * @param e Event created by the button press.
   * @param button The button that was pressed.
   */
  public void mousePressed(ViewportCluster vpc, Viewport vp, MouseEvent e, int button)
  {
  	// Get the bufferred image.
  	BufferedImage origImage = (BufferedImage)_origViewport.getImage();
  	
  	if (button == ViewportTool.LEFT_MOUSE_BUTTON && vpc != null) {
	    // Fill in the double array with the original data
      	double[][] data = new double[_height][_width];
	    Raster origRaster = origImage.getData();
 	    for (int j = 0; j < _height; j++) {
 	      for (int i = 0; i < _width; i++) {
  	      	data[j][i] = origRaster.getSampleDouble(i+_xStart, j+_yStart, 0);
	      }
	    }
      	// Get the blank image
	  	Iterator it = vpc.getViewports().iterator();
	  	Viewport blankViewport = (Viewport)it.next();
	  	blankViewport = (Viewport)it.next();
	    BufferedImage blankImage = (BufferedImage)blankViewport.getImage();
        // If showing the image intensity.
      	double[][] newData = new double[_height][_width];
      	// Find the maximum value
      	double maxData = 0.0;
      	for (int j = 0; j < _height; j++) {
        	for (int i = 0; i < _width; i++) {
          		if (data[j][i] > maxData) {
            		maxData = data[j][i];
          		}
          	}
        }
        // Normalization of data
      	for (int j = 0; j < _height; j++) {
        	for (int i = 0; i < _width; i++) {
          		newData[j][i] = 255.0 * data[j][i] / maxData;
          	}
        }
      	//set RGB value of imageCopy
      	for (int j = 0; j < _height; j++) {
        	for (int i = 0; i < _width; i++) {
          		int dataRGB = (0xff000000) | ((int)newData[j][i] << 16) |
                     ((int)newData[j][i] << 8) |
                     ((int)newData[j][i]);
          		blankImage.setRGB(i, j, dataRGB);
          	}
        }
        // Pass image data and bufferredimage to the _matrix.
        _matrix.initialization(blankImage, data);
        // If showing the image gradient.
        if (!_matrix.isIntensityImage) {
	    	newData = _matrix.getImageGradientArray();
	    	// Scaling of data
	      	for (int j = 0; j < _height; j++) {
	        	for (int i = 0; i < _width; i++) {
	          		newData[j][i] *= 255.0;
	          	}
	        }
	        //set RGB value of imageCopy
	      	for (int j = 0; j < _height; j++) {
	        	for (int i = 0; i < _width; i++) {
	          		int dataRGB = (0xff000000) | ((int)newData[j][i] << 16) |
	                     ((int)newData[j][i] << 8) |
	                     ((int)newData[j][i]);
	          		blankImage.setRGB(i, j, dataRGB);
	          	}
	        }
	    }
        // Repaint the Viewports and update the ViewportCluster background
	    vpc.repaint(blankViewport);
  	}
  	
  	// Change the box size
    else if (button == ViewportTool.RIGHT_MOUSE_BUTTON && vp == _origViewport) {
      if ((_width == 32) && (_height == 32))
      {
        _width = _imageWidth;
        _height = _imageHeight;
      }
      else
      {
        _width /= 2;
        _height /= 2;
      }
      _origViewport.removeAnnotation(_dyShape);
      _xStart = _cursorPoint.x - _width/2;
      _yStart = _cursorPoint.y - _height/2;
      // Set the origin of the rectangle zero if it is the size of the image.
      if ((_width == _imageWidth) && (_height == _imageHeight)) {
      	_xStart = 0;
      	_yStart = 0;	
      }
      Rectangle rect = new Rectangle(_xStart, _yStart, _width, _height);
      _dyShape = new DynamicAnnotationShape(rect);
      _dyShape.setForegroundColor(Color.yellow);
      _dyShape.setBackgroundColor(null);
      _origViewport.addAnnotation(_dyShape);
      vpc.repaint(vp);
    }	      
  }	// End Function
  
  
  /**
   * Invoked when a mouse is moved outside a ViewportCluster.
   *
   * @param vpc ViewportCluster which received the event.
   * @param vp Viewport which the mouse cursor was on during the event.
   * @param e Event created by the button press.
   */
  public void mouseExited(ViewportCluster vpc, Viewport vp, MouseEvent e)
  {
  	_origViewport.removeAnnotation(_dyShape);
  	vpc.repaint(_origViewport);
  }
}	// End Class